﻿using LibW4M.Data.Highscores;
using LibW4M.Data.Schemes;
using LibW4M.Data.Teams;
using LibW4M.Data.WeaponFactory;
using LibXom.Data;
using LibXom.Streams;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.PortableExecutable;
using System.Text;
using System.Threading.Tasks;

namespace LibW4M.Data.X
{
    public class DataBank : SaveDataEntry
    {
        public int Section;
        public IntResourceDetail[] IntResourceDetails;
        public UIntResourceDetail[] UIntResourceDetails;
        public StringResourceDetail[] StringResourceDetails;
        public FloatResourceDetail[] FloatResourceDetails;
        public VectorResourceDetail[] VectorResourceDetails;
        public ContainerResourceDetail[] ContainerResourceDetails;

        public DataBank(W4SaveFile fileBelongs, XomContainer mainContainer) : base(fileBelongs, mainContainer)
        {
        }

        public override void Load()
        {
            using (XomStreamReader reader = new XomStreamReader(new MemoryStream(this.mainContainer.GetData())))
            {
                reader.Skip(3);
                this.Section = reader.ReadCompressedInt();

                int[] intResourceCtnrs = reader.ReadCompressedIntArray();
                IntResourceDetails = new IntResourceDetail[intResourceCtnrs.Length];

                for (int i = 0; i < IntResourceDetails.Length; i++)
                    IntResourceDetails[i] = new IntResourceDetail(fileBelongs, fileBelongs.LookupContainerById(intResourceCtnrs[i]));

                int[] uintResourceCtnrs = reader.ReadCompressedIntArray();
                UIntResourceDetails = new UIntResourceDetail[uintResourceCtnrs.Length];
                
                for (int i = 0; i < UIntResourceDetails.Length; i++)
                    UIntResourceDetails[i] = new UIntResourceDetail(fileBelongs, fileBelongs.LookupContainerById(uintResourceCtnrs[i]));

                int[] stringResourceCtnrs = reader.ReadCompressedIntArray();
                StringResourceDetails = new StringResourceDetail[stringResourceCtnrs.Length];
                
                for (int i = 0; i < StringResourceDetails.Length; i++)
                    StringResourceDetails[i] = new StringResourceDetail(fileBelongs, fileBelongs.LookupContainerById(stringResourceCtnrs[i]));

                int[] floatResourceCtnrs = reader.ReadCompressedIntArray();
                FloatResourceDetails = new FloatResourceDetail[floatResourceCtnrs.Length];
                
                for (int i = 0; i < FloatResourceDetails.Length; i++)
                    FloatResourceDetails[i] = new FloatResourceDetail(fileBelongs, fileBelongs.LookupContainerById(floatResourceCtnrs[i]));

                int[] vectorResourceCtnrs = reader.ReadCompressedIntArray();
                VectorResourceDetails = new VectorResourceDetail[vectorResourceCtnrs.Length];
                
                for (int i = 0; i < VectorResourceDetails.Length; i++)
                    VectorResourceDetails[i] = new VectorResourceDetail(fileBelongs, fileBelongs.LookupContainerById(vectorResourceCtnrs[i]));

                int[] containerResourceCtnrs = reader.ReadCompressedIntArray();
                ContainerResourceDetails = new ContainerResourceDetail[containerResourceCtnrs.Length];
                
                for (int i = 0; i < ContainerResourceDetails.Length; i++)
                    ContainerResourceDetails[i] = new ContainerResourceDetail(fileBelongs, fileBelongs.LookupContainerById(containerResourceCtnrs[i]));

            }
        }

        public void writeResourceDetail(XomStreamWriter writer, SaveDataEntry[] resourceDetails)
        {
            int[] inArr = new int[resourceDetails.Length];

            for(int i = 0; i < resourceDetails.Length; i++)
            {
                resourceDetails[i].Save();
                inArr[i] = resourceDetails[i].containerId;
            }

            writer.WriteCompressedIntArray(inArr);
        }
        public override void Save()
        {
            using (MemoryStream ms = new MemoryStream())
            {
                using (XomStreamWriter writer = new XomStreamWriter(ms))
                {
                    writer.Skip(3);

                    writer.WriteCompressedInt(this.Section);

                    writeResourceDetail(writer, IntResourceDetails);
                    writeResourceDetail(writer, UIntResourceDetails);
                    writeResourceDetail(writer, StringResourceDetails);
                    writeResourceDetail(writer, FloatResourceDetails);
                    writeResourceDetail(writer, VectorResourceDetails);
                    writeResourceDetail(writer, ContainerResourceDetails);

                    ms.Seek(0x00, SeekOrigin.Begin);
                    mainContainer.SetData(ms.ToArray());
                }
            }
        }
    }
}
